package com.ruoyi.agentcenter.listener;

import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson2.JSONObject;
import com.google.common.eventbus.Subscribe;
import com.nsgk.agentcentersdk.entity.NSContractionEntity;
import com.ruoyi.agentcenter.domain.TAgentContraction;
import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.object.ContractionSession;
import com.ruoyi.agentcenter.object.Message;
import com.ruoyi.agentcenter.object.Result;
import com.ruoyi.agentcenter.service.ITAgentContractionService;
import com.ruoyi.agentcenter.service.ITAgentTaskService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.EventBusEngine;
import com.ruoyi.common.utils.sql.SqlUtil;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Date;

@Component
public class ContractionListener
{
    @Autowired
    private ITAgentContractionService itAgentContractionService;
    @Autowired
    private ITAgentTaskService itAgentTaskService;
    @Autowired
    private ISysDeptService    iSysDeptService;

    @PostConstruct
    public void init()
    {
        EventBusEngine.Register(this);
    }

    @Subscribe
    //@AllowConcurrentEvents
    public void handle(ContractionSession session)
    {
        TAgentContraction contraction = conv(session.message);
        Result result = session.result;

        SqlUtil.transaction(() -> {
            TAgentTask task = syncTask(session.message.getData(), contraction);
            contraction.setTaskId(task.getId());
            itAgentContractionService.insertTAgentContraction(contraction);
        });

        result.setData(contraction);
    }

    private TAgentContraction conv(Message<NSContractionEntity> message)
    {
        TAgentContraction contraction = new TAgentContraction();
        NSContractionEntity data = message.getData();
        contraction.setBookId(data.getBookId())
                .setDeptId(data.getDeptId())
                .setOutId(data.getOutId())
                .setOrgCode(data.getOrgCode())
                .setName(data.getName())
                .setBuildingTime(data.getBuildingTime())
                .setOrderAt(new Date())
                .setAgentStatus("1")
                .setOrderStatus("2")
                .normalized();
                ;
        return contraction;
    }

    private TAgentTask syncTask(NSContractionEntity entity, TAgentContraction contraction)
    {
        String orgCode = contraction.getOrgCode();
        TAgentTask task = new TAgentTask();
        task.setOrgCode(orgCode)
                .setOrderYear(contraction.getOrderYear())
                .setOrderMonth(contraction.getOrderMonth())
            ;
        TAgentTask tAgentTask = itAgentTaskService.selectTAgentTask(task);
        if(null == tAgentTask)
        {
            SysDept deptCond = new SysDept();
            deptCond.setOrgCode(orgCode);
            SysDept dept = iSysDeptService.selectSysDept(deptCond);
            Assert.notNull(dept, "部门不存在: {}", orgCode);
            task.setBookId(contraction.getBookId())
                    .setBookName(entity.getBookName())

                    .setContracCount(0)
                    .setVoucherCount(0)
                    .setAssetCount(0)
                    .setAllCount(0)
                    .setFinishCount(0)

                    .setAgentCenter(dept.getAgentCenter())
                    .setAgentStatus("1")
                ;
            itAgentTaskService.fillTAgentTaskOrg(task);
            task.setCreateBy("admin");
            itAgentTaskService.insertTAgentTask(task);
            tAgentTask = task;
        }

        task = new TAgentTask();
        task.setContracCount(1);
        task.setId(tAgentTask.getId());
        itAgentTaskService.updateTAgentTaskCount(task);

        return task;
    }
}
