package com.ruoyi.agentcenter.service.impl;

import com.nsgk.agentcentersdk.api.NSSDK;
import com.nsgk.agentcentersdk.api.NSSDKServer;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.core.NSReportObject;
import com.nsgk.agentcentersdk.entity.NSContractionEntity;
import com.nsgk.agentcentersdk.entity.NSEntity;
import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.err.NSException;
import com.ruoyi.agentcenter.object.ContractionSession;
import com.ruoyi.agentcenter.object.Message;
import com.ruoyi.agentcenter.object.Result;
import com.ruoyi.agentcenter.object.Session;
import com.ruoyi.agentcenter.service.IAgentCenter;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.utils.EventBusEngine;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;

@Service
public class AgentCenterImpl implements IAgentCenter
{

    public int getProtocol(HttpServletRequest request)
    {
        int protocol = NSSDKServer.GetProtocol(request);
        if(!NSProtocol.IsValid(protocol))
        {
            NSErrGlobal.Err(NSErrno.ERRNO_SYS_INVALID_PROTOCOL);
            throw new NSException(NSErrno.ERRNO_SYS_INVALID_PROTOCOL, String.format("未知的协议类型: 0x%x", protocol));
        }
        return protocol;
    }

    public Message<?> getMessage(HttpServletRequest request)
    {
        NSSDKServer server;
        NSReportObject<?> reportObject;
        Message<?> message;

        NSSDK.InitServer(RuoYiConfig.Secret.privateKey);
        server = NSSDK.InstanceServer();
        int protocol = getProtocol(request);
        Class<? extends NSEntity> clazz;
        switch(protocol)
        {
            case NSProtocol.NS_PROTOCOL_CONTRACTION:
                clazz = NSContractionEntity.class;
                break;
            default:
                return null;
        }
        reportObject = server.Recv(request, clazz);
        message = new Message<>(protocol, reportObject.getIdentifier(), reportObject.getTimestamp(), reportObject.getData());
        return message;
    }

    public Session<?, ?> createSession(Message<?> message)
    {
        Session<?, ?> session;
        int protocol;

        protocol = message.getProtocol();
        switch(protocol)
        {
            case NSProtocol.NS_PROTOCOL_CONTRACTION:
                session = new ContractionSession((Message<NSContractionEntity>) message);
                break;
            default:
                return null;
        }
        return session;
    }

    @Override
    public Result<?> postHandle(HttpServletRequest request)
    {
        Message<?> message = getMessage(request);
        Session<?, ?> session = createSession(message);
        EventBusEngine.Post(session);
        return session.result;
    }
}
